diff --git a/Marlin/Configuration_adv.h b/Marlin/Configuration_adv.h
index 4629224949..edf981e0d9 100644
--- a/Marlin/Configuration_adv.h
+++ b/Marlin/Configuration_adv.h
@@ -549,6 +549,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/default/Configuration_adv.h b/Marlin/src/config/default/Configuration_adv.h
index 4629224949..edf981e0d9 100644
--- a/Marlin/src/config/default/Configuration_adv.h
+++ b/Marlin/src/config/default/Configuration_adv.h
@@ -549,6 +549,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/AlephObjects/TAZ4/Configuration_adv.h b/Marlin/src/config/examples/AlephObjects/TAZ4/Configuration_adv.h
index c13082b085..52e2b10397 100644
--- a/Marlin/src/config/examples/AlephObjects/TAZ4/Configuration_adv.h
+++ b/Marlin/src/config/examples/AlephObjects/TAZ4/Configuration_adv.h
@@ -549,6 +549,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/Anet/A6/Configuration_adv.h b/Marlin/src/config/examples/Anet/A6/Configuration_adv.h
index f0cc32b929..f7ab76ebfb 100644
--- a/Marlin/src/config/examples/Anet/A6/Configuration_adv.h
+++ b/Marlin/src/config/examples/Anet/A6/Configuration_adv.h
@@ -549,6 +549,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/Anet/A8/Configuration_adv.h b/Marlin/src/config/examples/Anet/A8/Configuration_adv.h
index a3bdae8198..7fa232a7fb 100644
--- a/Marlin/src/config/examples/Anet/A8/Configuration_adv.h
+++ b/Marlin/src/config/examples/Anet/A8/Configuration_adv.h
@@ -549,6 +549,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/BQ/Hephestos/Configuration_adv.h b/Marlin/src/config/examples/BQ/Hephestos/Configuration_adv.h
index b0afce3ecd..d843173b1f 100644
--- a/Marlin/src/config/examples/BQ/Hephestos/Configuration_adv.h
+++ b/Marlin/src/config/examples/BQ/Hephestos/Configuration_adv.h
@@ -549,6 +549,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/BQ/WITBOX/Configuration_adv.h b/Marlin/src/config/examples/BQ/WITBOX/Configuration_adv.h
index b0afce3ecd..d843173b1f 100644
--- a/Marlin/src/config/examples/BQ/WITBOX/Configuration_adv.h
+++ b/Marlin/src/config/examples/BQ/WITBOX/Configuration_adv.h
@@ -549,6 +549,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/Cartesio/Configuration_adv.h b/Marlin/src/config/examples/Cartesio/Configuration_adv.h
index 9348e57dd1..623d74615d 100644
--- a/Marlin/src/config/examples/Cartesio/Configuration_adv.h
+++ b/Marlin/src/config/examples/Cartesio/Configuration_adv.h
@@ -549,6 +549,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/Creality/CR-10/Configuration_adv.h b/Marlin/src/config/examples/Creality/CR-10/Configuration_adv.h
index f1f82a3296..a2b539197d 100644
--- a/Marlin/src/config/examples/Creality/CR-10/Configuration_adv.h
+++ b/Marlin/src/config/examples/Creality/CR-10/Configuration_adv.h
@@ -549,6 +549,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/Felix/Configuration_adv.h b/Marlin/src/config/examples/Felix/Configuration_adv.h
index 93ddd2aba3..eaceb0dcc7 100644
--- a/Marlin/src/config/examples/Felix/Configuration_adv.h
+++ b/Marlin/src/config/examples/Felix/Configuration_adv.h
@@ -549,6 +549,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/Folger Tech/i3-2020/Configuration_adv.h b/Marlin/src/config/examples/Folger Tech/i3-2020/Configuration_adv.h
index 7e635eadf0..ad2f1e5c92 100644
--- a/Marlin/src/config/examples/Folger Tech/i3-2020/Configuration_adv.h
+++ b/Marlin/src/config/examples/Folger Tech/i3-2020/Configuration_adv.h
@@ -549,6 +549,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/Infitary/i3-M508/Configuration_adv.h b/Marlin/src/config/examples/Infitary/i3-M508/Configuration_adv.h
index aa0b535291..524b7d0e61 100644
--- a/Marlin/src/config/examples/Infitary/i3-M508/Configuration_adv.h
+++ b/Marlin/src/config/examples/Infitary/i3-M508/Configuration_adv.h
@@ -549,6 +549,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/Mks/Sbase/Configuration_adv.h b/Marlin/src/config/examples/Mks/Sbase/Configuration_adv.h
index 91e4f765a6..69d29f9f3f 100644
--- a/Marlin/src/config/examples/Mks/Sbase/Configuration_adv.h
+++ b/Marlin/src/config/examples/Mks/Sbase/Configuration_adv.h
@@ -549,6 +549,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/RigidBot/Configuration_adv.h b/Marlin/src/config/examples/RigidBot/Configuration_adv.h
index 999a159014..8105b9fcbc 100644
--- a/Marlin/src/config/examples/RigidBot/Configuration_adv.h
+++ b/Marlin/src/config/examples/RigidBot/Configuration_adv.h
@@ -549,6 +549,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/SCARA/Configuration_adv.h b/Marlin/src/config/examples/SCARA/Configuration_adv.h
index 787eb0b7e6..fdcce8c6fd 100644
--- a/Marlin/src/config/examples/SCARA/Configuration_adv.h
+++ b/Marlin/src/config/examples/SCARA/Configuration_adv.h
@@ -549,6 +549,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/Sanguinololu/Configuration_adv.h b/Marlin/src/config/examples/Sanguinololu/Configuration_adv.h
index 5c750f4d51..7f7e751b75 100644
--- a/Marlin/src/config/examples/Sanguinololu/Configuration_adv.h
+++ b/Marlin/src/config/examples/Sanguinololu/Configuration_adv.h
@@ -538,6 +538,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/TinyBoy2/Configuration_adv.h b/Marlin/src/config/examples/TinyBoy2/Configuration_adv.h
index 0dcd3e436d..8baa9d9916 100644
--- a/Marlin/src/config/examples/TinyBoy2/Configuration_adv.h
+++ b/Marlin/src/config/examples/TinyBoy2/Configuration_adv.h
@@ -549,6 +549,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/Velleman/K8400/Configuration_adv.h b/Marlin/src/config/examples/Velleman/K8400/Configuration_adv.h
index e24bd36c5c..7286846e05 100644
--- a/Marlin/src/config/examples/Velleman/K8400/Configuration_adv.h
+++ b/Marlin/src/config/examples/Velleman/K8400/Configuration_adv.h
@@ -549,6 +549,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/delta/FLSUN/auto_calibrate/Configuration_adv.h b/Marlin/src/config/examples/delta/FLSUN/auto_calibrate/Configuration_adv.h
index aaf54049eb..81ba827c12 100644
--- a/Marlin/src/config/examples/delta/FLSUN/auto_calibrate/Configuration_adv.h
+++ b/Marlin/src/config/examples/delta/FLSUN/auto_calibrate/Configuration_adv.h
@@ -551,6 +551,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/delta/FLSUN/kossel_mini/Configuration_adv.h b/Marlin/src/config/examples/delta/FLSUN/kossel_mini/Configuration_adv.h
index 0506b2d67a..7836249745 100644
--- a/Marlin/src/config/examples/delta/FLSUN/kossel_mini/Configuration_adv.h
+++ b/Marlin/src/config/examples/delta/FLSUN/kossel_mini/Configuration_adv.h
@@ -551,6 +551,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/delta/generic/Configuration_adv.h b/Marlin/src/config/examples/delta/generic/Configuration_adv.h
index 0506b2d67a..7836249745 100644
--- a/Marlin/src/config/examples/delta/generic/Configuration_adv.h
+++ b/Marlin/src/config/examples/delta/generic/Configuration_adv.h
@@ -551,6 +551,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/delta/kossel_mini/Configuration_adv.h b/Marlin/src/config/examples/delta/kossel_mini/Configuration_adv.h
index 0506b2d67a..7836249745 100644
--- a/Marlin/src/config/examples/delta/kossel_mini/Configuration_adv.h
+++ b/Marlin/src/config/examples/delta/kossel_mini/Configuration_adv.h
@@ -551,6 +551,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/delta/kossel_pro/Configuration_adv.h b/Marlin/src/config/examples/delta/kossel_pro/Configuration_adv.h
index 97770eb3f7..661bcd7deb 100644
--- a/Marlin/src/config/examples/delta/kossel_pro/Configuration_adv.h
+++ b/Marlin/src/config/examples/delta/kossel_pro/Configuration_adv.h
@@ -556,6 +556,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/delta/kossel_xl/Configuration_adv.h b/Marlin/src/config/examples/delta/kossel_xl/Configuration_adv.h
index dad3190adf..f11d829d49 100644
--- a/Marlin/src/config/examples/delta/kossel_xl/Configuration_adv.h
+++ b/Marlin/src/config/examples/delta/kossel_xl/Configuration_adv.h
@@ -551,6 +551,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/gCreate/gMax1.5+/Configuration_adv.h b/Marlin/src/config/examples/gCreate/gMax1.5+/Configuration_adv.h
index 0f377bec48..ddb4c3bc6a 100644
--- a/Marlin/src/config/examples/gCreate/gMax1.5+/Configuration_adv.h
+++ b/Marlin/src/config/examples/gCreate/gMax1.5+/Configuration_adv.h
@@ -549,6 +549,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/makibox/Configuration_adv.h b/Marlin/src/config/examples/makibox/Configuration_adv.h
index a6edb67e50..911fa693ec 100644
--- a/Marlin/src/config/examples/makibox/Configuration_adv.h
+++ b/Marlin/src/config/examples/makibox/Configuration_adv.h
@@ -549,6 +549,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/tvrrug/Round2/Configuration_adv.h b/Marlin/src/config/examples/tvrrug/Round2/Configuration_adv.h
index 798f84a4f5..1253be5f55 100644
--- a/Marlin/src/config/examples/tvrrug/Round2/Configuration_adv.h
+++ b/Marlin/src/config/examples/tvrrug/Round2/Configuration_adv.h
@@ -549,6 +549,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/config/examples/wt150/Configuration_adv.h b/Marlin/src/config/examples/wt150/Configuration_adv.h
index b04b59276e..2e354d3128 100644
--- a/Marlin/src/config/examples/wt150/Configuration_adv.h
+++ b/Marlin/src/config/examples/wt150/Configuration_adv.h
@@ -549,6 +549,9 @@
//#define LCD_PROGRESS_BAR_TEST
#endif
+ // Add an 'M73' G-code to set the current percentage
+ //#define LCD_SET_PROGRESS_MANUALLY
+
// This allows hosts to request long names for files and folders with M33
//#define LONG_FILENAME_HOST_SUPPORT
diff --git a/Marlin/src/gcode/gcode.cpp b/Marlin/src/gcode/gcode.cpp
index 880cf1b059..5c0502772a 100644
--- a/Marlin/src/gcode/gcode.cpp
+++ b/Marlin/src/gcode/gcode.cpp
@@ -354,6 +354,10 @@ void GcodeSuite::process_next_command() {
case 49: M49(); break; // M49: Turn on or off G26 debug flag for verbose output
#endif
+ #if ENABLED(ULTRA_LCD) && ENABLED(LCD_SET_PROGRESS_MANUALLY)
+ case 73: M73(); break; // M73: Set progress percentage (for display on LCD)
+ #endif
+
case 75: M75(); break; // M75: Start print timer
case 76: M76(); break; // M76: Pause print timer
case 77: M77(); break; // M77: Stop print timer
diff --git a/Marlin/src/gcode/gcode.h b/Marlin/src/gcode/gcode.h
index 39cc5a79b5..2320f838d2 100644
--- a/Marlin/src/gcode/gcode.h
+++ b/Marlin/src/gcode/gcode.h
@@ -437,6 +437,10 @@ private:
static void M49();
#endif
+ #if ENABLED(ULTRA_LCD) && ENABLED(LCD_SET_PROGRESS_MANUALLY)
+ static void M73();
+ #endif
+
static void M75();
static void M76();
static void M77();
diff --git a/Marlin/src/gcode/host/M115.cpp b/Marlin/src/gcode/host/M115.cpp
index e509ae6a95..99b4a81593 100644
--- a/Marlin/src/gcode/host/M115.cpp
+++ b/Marlin/src/gcode/host/M115.cpp
@@ -72,6 +72,13 @@ void GcodeSuite::M115() {
SERIAL_PROTOCOLLNPGM("Cap:LEVELING_DATA:0");
#endif
+ // BUILD_PERCENT (M73)
+ #if ENABLED(LCD_SET_PROGRESS_MANUALLY)
+ SERIAL_PROTOCOLLNPGM("Cap:BUILD_PERCENT:1");
+ #else
+ SERIAL_PROTOCOLLNPGM("Cap:BUILD_PERCENT:0");
+ #endif
+
// SOFTWARE_POWER (M80, M81)
#if HAS_POWER_SWITCH
SERIAL_PROTOCOLLNPGM("Cap:SOFTWARE_POWER:1");
diff --git a/Marlin/src/gcode/lcd/M73.cpp b/Marlin/src/gcode/lcd/M73.cpp
new file mode 100644
index 0000000000..478424318e
--- /dev/null
+++ b/Marlin/src/gcode/lcd/M73.cpp
@@ -0,0 +1,48 @@
+/**
+ * Marlin 3D Printer Firmware
+ * Copyright (C) 2016 MarlinFirmware [https://github.com/MarlinFirmware/Marlin]
+ *
+ * Based on Sprinter and grbl.
+ * Copyright (C) 2011 Camiel Gubbels / Erik van der Zalm
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see .
+ *
+ */
+
+#include "../../inc/MarlinConfig.h"
+
+#if ENABLED(ULTRA_LCD) && ENABLED(LCD_SET_PROGRESS_MANUALLY)
+
+#include "../gcode.h"
+#include "../../module/parser.h"
+#include "../../lcd/ultralcd.h"
+#include "../../sd/cardreader.h"
+
+/**
+ * M73: Set percentage complete (for display on LCD)
+ *
+ * Example:
+ * M73 P25 ; Set progress to 25%
+ *
+ * Notes:
+ * This has no effect during an SD print job
+ */
+void GcodeSuite::M73() {
+ if (!IS_SD_PRINTING && parser.seen('P')) {
+ progress_bar_percent = parser.value_byte();
+ NOMORE(progress_bar_percent, 100);
+ }
+}
+
+#endif // ULTRA_LCD && LCD_SET_PROGRESS_MANUALLY
diff --git a/Marlin/src/lcd/ultralcd.h b/Marlin/src/lcd/ultralcd.h
index 16b3998171..ef9bc7c2b7 100644
--- a/Marlin/src/lcd/ultralcd.h
+++ b/Marlin/src/lcd/ultralcd.h
@@ -63,6 +63,10 @@
void dontExpireStatus();
#endif
+ #if ENABLED(LCD_SET_PROGRESS_MANUALLY)
+ extern uint8_t progress_bar_percent;
+ #endif
+
#if ENABLED(ADC_KEYPAD)
uint8_t get_ADC_keyValue();
#endif
diff --git a/Marlin/src/lcd/ultralcd_impl_DOGM.h b/Marlin/src/lcd/ultralcd_impl_DOGM.h
index f9f1abcfe1..7e38127c15 100644
--- a/Marlin/src/lcd/ultralcd_impl_DOGM.h
+++ b/Marlin/src/lcd/ultralcd_impl_DOGM.h
@@ -536,7 +536,7 @@ static void lcd_implementation_status_screen() {
// SD Card Symbol
//
- if (PAGE_CONTAINS(42 - (TALL_FONT_CORRECTION), 51 - (TALL_FONT_CORRECTION))) {
+ if (card.isFileOpen() && PAGE_CONTAINS(42 - (TALL_FONT_CORRECTION), 51 - (TALL_FONT_CORRECTION))) {
// Upper box
u8g.drawBox(42, 42 - (TALL_FONT_CORRECTION), 8, 7); // 42-48 (or 41-47)
// Right edge
@@ -560,7 +560,11 @@ static void lcd_implementation_status_screen() {
PROGRESS_BAR_WIDTH, 4 - (TALL_FONT_CORRECTION)
);
- if (IS_SD_PRINTING) {
+ #if DISABLED(LCD_SET_PROGRESS_MANUALLY)
+ const uint8_t progress_bar_percent = card.percentDone();
+ #endif
+
+ if (progress_bar_percent > 1) {
//
// Progress bar solid part
@@ -569,7 +573,7 @@ static void lcd_implementation_status_screen() {
if (PAGE_CONTAINS(50, 51 - (TALL_FONT_CORRECTION))) // 50-51 (or just 50)
u8g.drawBox(
PROGRESS_BAR_X + 1, 50,
- (uint16_t)((PROGRESS_BAR_WIDTH - 2) * card.percentDone() * 0.01), 2 - (TALL_FONT_CORRECTION)
+ (uint16_t)((PROGRESS_BAR_WIDTH - 2) * progress_bar_percent * 0.01), 2 - (TALL_FONT_CORRECTION)
);
//
@@ -580,7 +584,7 @@ static void lcd_implementation_status_screen() {
if (PAGE_CONTAINS(41, 48)) {
// Percent complete
u8g.setPrintPos(55, 48);
- u8g.print(itostr3(card.percentDone()));
+ u8g.print(itostr3(progress_bar_percent));
u8g.print('%');
}
#endif
diff --git a/Marlin/src/lcd/ultralcd_impl_HD44780.h b/Marlin/src/lcd/ultralcd_impl_HD44780.h
index 7cf64789b8..adfa5b8fa3 100644
--- a/Marlin/src/lcd/ultralcd_impl_HD44780.h
+++ b/Marlin/src/lcd/ultralcd_impl_HD44780.h
@@ -203,9 +203,9 @@ extern volatile uint8_t buttons; //an extended version of the last checked butt
#include "utf_mapper.h"
#if ENABLED(LCD_PROGRESS_BAR)
- static millis_t progress_bar_ms = 0;
+ static millis_t progress_bar_ms = 0; // Start millis of the current progress bar cycle
#if PROGRESS_MSG_EXPIRE > 0
- static millis_t expire_status_ms = 0;
+ static millis_t expire_status_ms = 0; // millis at which to expire the status message
#endif
#define LCD_STR_PROGRESS "\x03\x04\x05"
#endif
@@ -842,12 +842,13 @@ static void lcd_implementation_status_screen() {
#if ENABLED(LCD_PROGRESS_BAR)
+ #if DISABLED(LCD_SET_PROGRESS_MANUALLY)
+ const uint8_t progress_bar_percent = card.percentDone();
+ #endif
// Draw the progress bar if the message has shown long enough
// or if there is no message set.
- if (card.isFileOpen() && (ELAPSED(millis(), progress_bar_ms + PROGRESS_BAR_MSG_TIME) || !lcd_status_message[0])) {
- const uint8_t percent = card.percentDone();
- if (percent) return lcd_draw_progress_bar(percent);
- }
+ if (progress_bar_percent > 2 && (ELAPSED(millis(), progress_bar_ms + PROGRESS_BAR_MSG_TIME) || !lcd_status_message[0]))
+ return lcd_draw_progress_bar(progress_bar_percent);
#elif ENABLED(FILAMENT_LCD_DISPLAY) && ENABLED(SDSUPPORT)
diff --git a/Marlin/src/lcd/ultralcd.cpp b/Marlin/src/lcd/ultralcd.cpp
index dc9921f1d1..f9968e2798 100644
--- a/Marlin/src/lcd/ultralcd.cpp
+++ b/Marlin/src/lcd/ultralcd.cpp
@@ -83,6 +83,10 @@ char lcd_status_message[3 * (LCD_WIDTH) + 1] = WELCOME_MSG; // worst case is kan
uint8_t status_scroll_pos = 0;
#endif
+#if ENABLED(LCD_SET_PROGRESS_MANUALLY)
+ uint8_t progress_bar_percent;
+#endif
+
#if ENABLED(DOGLCD)
#include "ultralcd_impl_DOGM.h"
#include
@@ -604,36 +608,52 @@ void lcd_status_screen() {
#endif
#if ENABLED(LCD_PROGRESS_BAR)
- millis_t ms = millis();
- #if DISABLED(PROGRESS_MSG_ONCE)
- if (ELAPSED(ms, progress_bar_ms + PROGRESS_BAR_MSG_TIME + PROGRESS_BAR_BAR_TIME)) {
- progress_bar_ms = ms;
- }
+
+ //
+ // HD44780 implements the following message blinking and
+ // message expiration because Status Line and Progress Bar
+ // share the same line on the display.
+ //
+
+ // Set current percentage from SD when actively printing
+ #if ENABLED(LCD_SET_PROGRESS_MANUALLY)
+ if (IS_SD_PRINTING)
+ progress_bar_percent = card.percentDone();
#endif
+
+ millis_t ms = millis();
+
+ // If the message will blink rather than expire...
+ #if DISABLED(PROGRESS_MSG_ONCE)
+ if (ELAPSED(ms, progress_bar_ms + PROGRESS_BAR_MSG_TIME + PROGRESS_BAR_BAR_TIME))
+ progress_bar_ms = ms;
+ #endif
+
#if PROGRESS_MSG_EXPIRE > 0
+
// Handle message expire
if (expire_status_ms > 0) {
- #if ENABLED(SDSUPPORT)
- if (card.isFileOpen()) {
- // Expire the message when printing is active
- if (IS_SD_PRINTING) {
- if (ELAPSED(ms, expire_status_ms)) {
- lcd_status_message[0] = '\0';
- expire_status_ms = 0;
- }
- }
- else {
- expire_status_ms += LCD_UPDATE_INTERVAL;
- }
- }
- else {
+
+ #if DISABLED(LCD_SET_PROGRESS_MANUALLY)
+ const uint8_t progress_bar_percent = card.percentDone();
+ #endif
+
+ // Expire the message if a job is active and the bar has ticks
+ if (progress_bar_percent > 2 && !print_job_timer.isPaused()) {
+ if (ELAPSED(ms, expire_status_ms)) {
+ lcd_status_message[0] = '\0';
expire_status_ms = 0;
}
- #else
- expire_status_ms = 0;
- #endif // SDSUPPORT
+ }
+ else {
+ // Defer message expiration before bar appears
+ // and during any pause (not just SD)
+ expire_status_ms += LCD_UPDATE_INTERVAL;
+ }
}
- #endif
+
+ #endif // PROGRESS_MSG_EXPIRE
+
#endif // LCD_PROGRESS_BAR
#if ENABLED(ULTIPANEL)
@@ -2860,7 +2880,7 @@ void kill_screen(const char* lcd_msg) {
#if ENABLED(MAX_SOFTWARE_ENDSTOP_Z)
max = soft_endstop_max[Z_AXIS];
#endif
- break;
+ default: break;
}
#endif // MIN_SOFTWARE_ENDSTOPS || MAX_SOFTWARE_ENDSTOPS